/********************************************************
 * Data Visualization - A Series of Unfortunate Events  *
 * Tested in Processing 3                               *     
 *                                                      *
 * Author: Sarah Wells                                  *
 * Supervisor: George Legrady                           *
 *                                                      *
 * Purpose: Data visualization by title and month for   *
 *          A Series of Unfortunate Events              *
 *                                                      *
 *                                                      *
 * Usage:   No key: Overview by Title per month ('o')   *
 *          'a' -1, 'b' -2, 'c' -3, 'd' -4, 'e' -5,     *
 *          'f' -6, 'g' -7, 'h'-8, 'i' -9, 'j' -10,     *
 *          'k' -11, 'l' -12, 'm' - 13 (Book Numbers)   *
 *          't' Month cummulative Titles                *
  *         arrows to move through Books or Months      *
 ********************************************************/

// Layout variables 
float constantHorizontalMargin = 160;
float constantVerticalMargin = 80;
float placeholder = 0;
int booknum = 14, monthnum = -1;

// width and height for each cell
float cellWidth, cellHeight, standardCellWidth, otherCellHeight, otherCellWidth, gridWidth, gridHeight, bookMaxHeight;

// 2D matrix table related variables
Table table;

int numRows, numColumns;
float[][] dataMatrix;
float maxValue=0, minValue=5000, maxMonth=0;
float[] maxMonthBook;

String[] titles;
String[] dateym;

PImage colorBar;
String[] colorPath = {"darkerrainbow.jpg"};


PImage[] books;

/*
int[][] positions = 
{{0,0},
{5,1},
{8,3},
{10,7},
{8,12},
{4,14},
{5,19},
{10,18},
{15,16},
{19,13},
{24,14},
{27,17},
{29,21}
};

*/




//diagonal 
int[][] positions = 
{{0,0},
{3,2},
{6,4},
{9,6},
{12,8},
{15,10},
{18,12},
{21,14},
{24,16},
{27,18},
{30,20},
{33,22},
{36,24}
};


/*
bookcolor =     R: 219 G: 187 B: 124
1 bad beginning =   R: 43 G: 55 B: 78
2 reptile room = R: 57 G: 12 B: 39
3 wide window =   R: 50 G: 62 B: 64
4 miserable mill =   R: 113 G: 97 B: 136
5 austere =   R: 84 G: 84 B: 86
6 ersatz = R: 212 G: 97 B: 104
7 vile = R: 91 G: 102 B: 88
8 hostile =   R: 201 G: 124 B: 96
9 carnivorous =   R: 60 G: 143 B: 185
10 slippery = R: 119 G: 99 B: 48
11 grim = R: 166 G: 211 B: 186
12 penultimate = R: 233 G: 69 B: 41
13 end = R: 167 G: 145 B: 108
*/

int[][] bookcolors = {{249,220,180},
{43,55,78},
{57,12,39},
{50,62,64},
{113,97,136},
{84,84,86},
{212, 97, 104},
{91, 102, 88},
{201, 124, 96},
{60, 143, 185},
{119, 99, 48},
{166, 211, 186},
{233, 69, 41},
{167, 145, 108}
};

boolean colorScheme = false;

PFont fontBold, fontReg;
String[] fontlist = PFont.list();



void setup(){
  size(1280, 800); // setup the size of the window
  // In Processing 3, the size() function must be the first line of code inside setup()
  // and needs to use numbers instead of variables. 

  // Make the screen resizable.                     
  //surface.setResizable(true);
  
  
  //check available fonts
  //println(fontlist);
  fontBold = createFont("TimesNewRomanPS-BoldMT.tff", 10);
  fontReg = createFont("TimesNewRomanPSMT.tff", 10);
  //set reg font
  textFont(fontReg, 10);

  
  //Put data into Processing table from csv
  table = loadTable("TitleYearMonth Pivot Table 1.csv");
  
  // Get row and column information
  numRows = table.getRowCount(); 
  numColumns = table.getColumnCount();
  println("Rows: " + numRows + " Columns: " + numColumns);
  
  //Rows - 17
  //Cols -38
  
  // Create a 2D array to receive data from the table 
  dataMatrix = new float[numRows-3][numColumns-2]; // doesn't need the first two columns, first three rows
  titles = new String[numRows-4];
  dateym = new String[numColumns-3];
  maxMonthBook = new float[numRows-3];
  books = new PImage[13];
  
  for(int i=0; i<numRows-3; i++){
    for(int j=0; j<numColumns-2; j++){
      dataMatrix[i][j] = table.getFloat(i+3,j+2);
     
      //Find the max/min value of the whole data except totals in last rows/columns
      if(dataMatrix[i][j] > maxValue && i<numRows-4 && j<numColumns-3){
        maxValue = dataMatrix[i][j];
      }
      if(dataMatrix[i][j] < minValue && i<numRows-4 && j<numColumns-3){
        minValue = dataMatrix[i][j];
      }
      if(dataMatrix[i][j] > maxMonth && j<numColumns-3){
        maxMonth = dataMatrix[i][j];
      }
      if(dataMatrix[i][j] > maxMonthBook[i] && i<numRows-4 && j<numColumns-3){
        maxMonthBook[i] = dataMatrix[i][j];
      }
    }
    //check max each book per month worked
    if(i<numRows-4){
    println("Max Month #" + (i+1) + ": " + maxMonthBook[i]);
    }
  }
  
  println("Max Value is " + maxValue);
  println("Min Value is " + minValue);
  //max = 56
  //min = 0
  
  //check total overall count
  println("Total overall checkouts: " + dataMatrix[numRows-4][numColumns-3]);
  //5683, correct
  
  //check max count per Month total overall count
    
  println("Total checkout in Max Month: " + maxMonth);
  //297, correct
  
    //modifying the cell size according to window size WAY ONE
  otherCellWidth = (width - constantHorizontalMargin*2)/(numColumns-3);
  standardCellWidth = (width - constantHorizontalMargin*2)/(numRows-4);
  otherCellHeight = (height - constantVerticalMargin*2)/(numRows-4);
  
  //grid 32*24
  gridWidth = (width - constantHorizontalMargin*2)/32;
  gridHeight = (height - constantVerticalMargin*2)/24;
  bookMaxHeight = (height - constantVerticalMargin*2)/10;
  
  //load book images
  for(int i=0; i<13; i++){
    if(i<9){
    books[i]= loadImage("0" + (i+1) + ".jpeg");
    }
    else{
      books[i]= loadImage((i+1) + ".jpeg");
    }
  }
  
  //modifying the cell size according to window size WAY TWO
  //cellWidth = (width - constantHorizontalMargin*2)/(numRows-3);
  cellWidth = (width - constantHorizontalMargin*2)/maxMonth;
  cellHeight = (height - constantVerticalMargin*3)/(numColumns-4);
  
    
    // put title and date in year month format data from the table to the array, print to check
    for(int i=0; i<numRows-4; i++){
    titles[i] = table.getString(i+3, 0);
      println("Title #" + (i+1) + " " + titles[i]);
    }
    for(int i=0; i<numColumns-3; i++){
    dateym[i] = table.getString(1, i+2);
    println("Date #" + (i+1) + " " + dateym[i]);
    }
  
  
}

void draw(){
  background(255);
  if(monthnum != -1){
    
  //bookcolored background
  //background(bookcolors[0][0],bookcolors[0][1], bookcolors[0][2]);
  background(0);
  }
  //backdrop of image white will have to modify for different views
  //fill(255);
  //rect(constantHorizontalMargin+60,constantVerticalMargin, 1280-2*constantHorizontalMargin, 736-2*constantVerticalMargin);
  
  translate(60, 0);
  colorBar = loadImage(colorPath[0]);
  
  /*
    if(monthnum != -1){
    //draw lines
  for(int i=0;i<12; i++){
    fill(0);
    stroke(20);
  line(constantHorizontalMargin + gridWidth*positions[i][0], constantHorizontalMargin + gridHeight*positions[i][1],
  constantHorizontalMargin + gridWidth*positions[i+1][0], constantHorizontalMargin + gridHeight*positions[i+1][1]);
  }
    }
     */
     // FIRST WAY
   // shading table
  //Draw 2D Matrix
  
  if(booknum==14&&monthnum==-1){
  for(int i=0; i<dataMatrix.length-1; i++){
    for(int j=0; j<dataMatrix[0].length-1; j++){
      noStroke();
      color c = colorBar.get((int)((colorBar.width-2)*(i+1)/13), colorBar.height/2);
      fill(c, (int)255*dataMatrix[i][j]/maxValue);
      //color like the book:
      //fill(bookcolors[i+1][0],bookcolors[i+1][1],bookcolors[i+1][2], 50+(int)205*dataMatrix[i][j]/maxValue);
      rect(constantHorizontalMargin + otherCellWidth*j, constantVerticalMargin 
           + otherCellHeight*i, otherCellWidth, otherCellHeight);
    }
  }
  
    //Draw Title
  
  textAlign(CENTER, CENTER);
  textSize(16);
  fill(0);
  text("A Series of Unfortunate Events: Popularity By Month", width/2, constantVerticalMargin/2);

//side labels
  for(int i=0; i<titles.length; i++){
    //date year month format
    textAlign(RIGHT, CENTER);
          color c = colorBar.get((int)((colorBar.width-2)*(i+1)/13), colorBar.height/2);

    fill(c);
    textSize(10);
          //color like the book:
      //fill(bookcolors[i+1][0],bookcolors[i+1][1],bookcolors[i+1][2]);
      if(mouseX > constantHorizontalMargin-30 && mouseX < constantHorizontalMargin
      && mouseY > (constantVerticalMargin +otherCellHeight*i) && mouseY <(constantVerticalMargin +otherCellHeight*(1+i))){
        textSize(16);
      }
      
    
    text(titles[i], constantHorizontalMargin-3, constantVerticalMargin +otherCellHeight*(0.5+i));
    
    
     //show total number on the right side
 fill(0);
    textAlign(LEFT, CENTER);
    text((int)dataMatrix[i][numColumns-3], width-constantHorizontalMargin+3,constantVerticalMargin +otherCellHeight*(0.5+i));
  }
  
  //bottom slanted labels
  
  
  for(int i=0; i<numColumns-3; i++){ 
    
 fill(0);
    
    textAlign(CENTER, TOP);
    textSize(12);
    translate(constantHorizontalMargin + otherCellWidth*i - 2, constantVerticalMargin + otherCellHeight*(numRows-3)-10);
    rotate(-1);
    text(dateym[i], 0, 0);
    rotate(1);
    translate(-(constantHorizontalMargin + otherCellWidth*i - 2), -(constantVerticalMargin + otherCellHeight*(numRows-3)-10));
  
}
  
  }
  else if(monthnum==-1&&booknum!=14){
  
  // SECOND WAY
  //cell based on book
  
  //Draw 2D Matrix
  for(int j=0; j<dataMatrix[0].length-1; j++){
    placeholder=0;
     for(int i=0; i<dataMatrix.length-1; i++){
     if(booknum!=0){
        i+=(booknum-1);
        cellWidth = (width - constantHorizontalMargin*2)/maxValue;
        //fill(5*(i-7)^2,255-5*(i-7)^2,(int) pow(1.4,i), (int)255*dataMatrix[i][j]/maxMonthBook[i]);
      }
      else{
         cellWidth = (width - constantHorizontalMargin*2)/maxMonth;
                  //fill((5*pow((i-7),2)),255 - (5*pow((i-7),2)), (int)pow(2,i) , (int)255*dataMatrix[i][j]/maxValue);

         //fill(5*(i-7)^2,255-5*(i-7)^2,(int) pow(1.4,i), (int)255*dataMatrix[i][j]/maxValue);
      }
      noStroke();
      strokeWeight(1);
      
      
        color c = colorBar.get((int)((colorBar.width-2)*(i+1)/13), colorBar.height/2);
  
        if(booknum!=0){
        fill(c, (int)255*dataMatrix[i][j]/maxMonthBook[i]);
        //color like the book
        //fill(bookcolors[i+1][0],bookcolors[i+1][1],bookcolors[i+1][2], 50+(int)205*dataMatrix[i][j]/maxMonthBook[i]);
        
        }
        else{
        fill(c);
        if(mouseX>(constantHorizontalMargin)
        && mouseX < (1280-constantHorizontalMargin)
        && mouseY > (constantVerticalMargin + cellHeight*j)
        && mouseY < (constantVerticalMargin + cellHeight*(j+1))
        ){
          fill(c, 100);
        }
        //color like the book
        //fill(bookcolors[i+1][0],bookcolors[i+1][1],bookcolors[i+1][2]);
        }
        
        //sets to book per key pressed and rescales

      
      /*
      if(!colorScheme){
        float greyScale = map(dataMatrix[i][j], minValue, maxValue, 255, 0);
        fill(greyScale);
      }
      else{
        color c = colorBar.get((int)map(dataMatrix[i][j], minValue, maxValue, 2, colorBar.width-2), colorBar.height/2);
        fill(c);
      }
      
      */
      rect(constantHorizontalMargin + placeholder, constantVerticalMargin 
           + cellHeight*j, cellWidth*dataMatrix[i][j], cellHeight);
      placeholder += cellWidth*dataMatrix[i][j];
              
        //ends loop, only the one book
      if(booknum!=0){
        i=dataMatrix.length;
      }
    }
  }
  
  

  
    
//bottom slanted labels
  
if(booknum==0){
  
  for(int i=0; i<numRows-4; i++){ 
    
    color c = colorBar.get((int)((colorBar.width-2)*(i+1)/13), colorBar.height/2);
    fill(c);
            //color like the book
        //fill(bookcolors[i+1][0],bookcolors[i+1][1],bookcolors[i+1][2]);
    
    textAlign(CENTER, TOP);
    textSize(12);
    translate(constantHorizontalMargin + standardCellWidth*i - 2, constantVerticalMargin +cellHeight*(numColumns-3) + 60);
    rotate(-1);
    text(titles[i], 0, 0);
    rotate(1);
    translate(-(constantHorizontalMargin + standardCellWidth*i - 2), -(constantVerticalMargin +cellHeight*(numColumns-3) + 60));
  }
  
}
   
  //Draw vertical labels

      
  for(int j=0; j<dateym.length; j++){
    //date year month format
    textAlign(RIGHT, CENTER);
    fill(0);
    textSize(10);
    
            if(mouseX>(constantHorizontalMargin)
        && mouseX < (1280-constantHorizontalMargin)
        && mouseY > (constantVerticalMargin + cellHeight*j)
        && mouseY < (constantVerticalMargin + cellHeight*(j+1))
        ){
          textSize(16);
        }
        
    text(dateym[j], constantHorizontalMargin-3, constantVerticalMargin +cellHeight*(0.5+j));
    
    
     //show total number on the right side
         if(booknum ==0){
    textAlign(LEFT, CENTER);
    text((int)dataMatrix[numRows-4][j], width-constantHorizontalMargin+3,constantVerticalMargin +cellHeight*(0.5+j));
         }
         else{
           textAlign(LEFT, CENTER);
           //textFont('Arial Bold');
    text((int)dataMatrix[booknum-1][j], width-constantHorizontalMargin+3,constantVerticalMargin +cellHeight*(0.5+j));
         }
  }

  
 
  //Draw Title
  
  textAlign(CENTER, CENTER);
  textSize(16);
  fill(0);
  if(booknum ==0){
  text("A Series of Unfortunate Events: Popularity By Month", width/2, constantVerticalMargin/2);
  }
  else{
    
   color c = colorBar.get((int)((colorBar.width-2)*(booknum)/13), colorBar.height/2);
    fill(c);
            //color like the book
        //fill(bookcolors[booknum][0],bookcolors[booknum][1],bookcolors[booknum][2]);

    text(titles[booknum-1], width/2, constantVerticalMargin/2);
  }
  
  
  
  }
  else{
    //do stuff for monthnum
    placeholder=0;
    
    //bookMaxHeight
    
   
    
     for(int i=0; i<dataMatrix.length-1; i++){
       
                cellWidth = (width - constantHorizontalMargin*2)/maxMonth;
                
                  //fill((5*pow((i-7),2)),255 - (5*pow((i-7),2)), (int)pow(2,i) , (int)255*dataMatrix[i][j]/maxValue);

         //fill(5*(i-7)^2,255-5*(i-7)^2,(int) pow(1.4,i), (int)255*dataMatrix[i][j]/maxValue);
      noStroke();
      /*position equations
       image(books[i], constantHorizontalMargin + placeholder, constantVerticalMargin 
           , (int) bookMaxHeight*dataMatrix[i][monthnum]/(1.5*dataMatrix[numRows-4][monthnum]/13), 
           (int) bookMaxHeight*dataMatrix[i][monthnum]/(dataMatrix[numRows-4][monthnum]/13));
           
            textAlign(CENTER, TOP);
    textSize(12);
    translate(constantHorizontalMargin + placeholder, constantVerticalMargin +cellHeight*(numColumns-3) -280);
    rotate(-1);
    text(titles[i], 0, 0);
    rotate(1);
    translate(-(constantHorizontalMargin + placeholder), -(constantVerticalMargin +cellHeight*(numColumns-3) -280));
    
      placeholder += bookMaxHeight*dataMatrix[i][monthnum]/(1.5*dataMatrix[numRows-4][monthnum]/13)+5;

*/

/*

image(books[i], constantHorizontalMargin + gridWidth*positions[i][0], constantVerticalMargin + gridWidth*positions[i][1]
           , (int) bookMaxHeight*dataMatrix[i][monthnum]/(1.5*dataMatrix[numRows-4][monthnum]/13), 
           (int) bookMaxHeight*dataMatrix[i][monthnum]/(dataMatrix[numRows-4][monthnum]/13));
           
           */
           
           
           /*
           
image(books[i], constantHorizontalMargin + i*standardCellWidth, 680-36*bookMaxHeight*dataMatrix[i][monthnum]/dataMatrix[numRows-4][monthnum]
           , (int) bookMaxHeight,
           (int) bookMaxHeight*3/2);
           */
           
           image(books[i], constantHorizontalMargin + i*standardCellWidth, 680-640*dataMatrix[i][monthnum]/56,
           (int) bookMaxHeight*dataMatrix[i][monthnum]/(1.5*dataMatrix[numRows-4][monthnum]/10),
           (int) bookMaxHeight*dataMatrix[i][monthnum]/(dataMatrix[numRows-4][monthnum]/10));
           
           
           
           
          
      

       /*
         cellWidth = (width - constantHorizontalMargin*2)/maxMonth;
                  //fill((5*pow((i-7),2)),255 - (5*pow((i-7),2)), (int)pow(2,i) , (int)255*dataMatrix[i][j]/maxValue);

         //fill(5*(i-7)^2,255-5*(i-7)^2,(int) pow(1.4,i), (int)255*dataMatrix[i][j]/maxValue);
      noStroke();
      
      
        color c = colorBar.get((int)((colorBar.width-2)*(i+1)/13), colorBar.height/2);
        fill(c);
        




      rect(constantHorizontalMargin + placeholder, constantVerticalMargin 
           , cellWidth*dataMatrix[i][monthnum], cellHeight*(numRows-4));
           
           
           
           textAlign(CENTER, TOP);
    textSize(12);
    translate(constantHorizontalMargin + placeholder, constantVerticalMargin +cellHeight*(numColumns-3) -280);
    rotate(-1);
    text(titles[i], 0, 0);
    rotate(1);
    translate(-(constantHorizontalMargin + placeholder), -(constantVerticalMargin +cellHeight*(numColumns-3) -280));
    
      placeholder += cellWidth*dataMatrix[i][monthnum];
      
      */
              
    }
    
      //Draw Title
  
  textAlign(CENTER, CENTER);
  textSize(20);
  fill(255);
  text("A Series of Unfortunate Events Popularity: " + dateym[monthnum] + " Total: " + dataMatrix[numRows-4][monthnum], width/2, constantVerticalMargin/2);

  
      /*
      for(int i=0; i<numRows-4; i++){ 
    
    color c = colorBar.get((int)((colorBar.width-2)*(i+1)/13), colorBar.height/2);
    fill(c);
    
    textAlign(CENTER, TOP);
    textSize(12);
    translate(constantHorizontalMargin + standardCellWidth*i - 2, constantVerticalMargin +cellHeight*(numColumns-3) -260);
    rotate(-1);
    text(titles[i], 0, 0);
    rotate(1);
    translate(-(constantHorizontalMargin + standardCellWidth*i - 2), -(constantVerticalMargin +cellHeight*(numColumns-3) -260));
  }
  */
  }
}


void mouseClicked(){
  if(monthnum==-1 && booknum==0 && mouseX > constantHorizontalMargin && mouseX < 1280 - constantHorizontalMargin/2){
    
  for(int j=0; j<dateym.length; j++){

    if(mouseY > (constantVerticalMargin + cellHeight*j) && mouseY <= (constantVerticalMargin +cellHeight*(1+j))){
      monthnum=j;
    }
    
  }
  }
  else if(booknum==14){
    for(int i=0; i<13; i++){
          if(mouseX > constantHorizontalMargin-30 && mouseX < constantHorizontalMargin
      && mouseY > (constantVerticalMargin +otherCellHeight*i) && mouseY <(constantVerticalMargin +otherCellHeight*(1+i))){
        booknum = i+1;
      }
    }
  }
  else{
    monthnum=-1;
    booknum=0;
  }
}


void keyPressed(){
  if(monthnum>-1 && monthnum <34 && key == CODED && (keyCode == UP || keyCode == RIGHT)){
    monthnum +=1;
  }
  else if(monthnum>0 && monthnum <35 && key == CODED && (keyCode == DOWN || keyCode == LEFT)){
    monthnum -=1;
  }
  else if(monthnum==-1 && booknum >0  && booknum <13 && key == CODED && (keyCode == UP || keyCode == RIGHT)){
    booknum +=1;
  }
  else if(monthnum==-1 && booknum >1 && booknum <=13 && key == CODED && (keyCode == DOWN || keyCode == LEFT)){
    booknum -=1;
  }
  else{
  monthnum=-1;
  if(key == 'a'){
    booknum = 1;
  }
    else if(key == 'b'){
    booknum = 2;
  }
    else if(key == 'c'){
    booknum = 3;
  }
    else if(key == 'd'){
    booknum = 4;
  }
    else if(key == 'e'){
    booknum = 5;
  }
    else if(key == 'f'){
    booknum = 6;
  }
    else if(key == 'g'){
    booknum = 7;
  }
    else if(key == 'h'){
    booknum = 8;
  }
    else if(key == 'i'){
    booknum = 9;
  }
    else if(key == 'j'){
    booknum = 10;
  }
    else if(key == 'k'){
    booknum = 11;
  }
    else if(key == 'l'){
    booknum = 12;
  }
    else if(key == 'm'){
    booknum = 13;
  }
  else if(key == 'o'){
    booknum=14;
  }
  else if(key =='t'){
    booknum=0;
  }
  }
}
